"""
Class that implements the common functionality for treatment of the standard output
"""

from optparse import OptionGroup
from os import path

class CommonStandardOutput(object):
    """ The class that defines options for standard output
    """

    def addOptions(self,logname=None):
        grp=OptionGroup(self.parser,
                        "Standard Output",
                        "Treatment of the standard output that is captured from the OpenFOAM-application")
        grp.add_option("--progress",
                       action="store_true",
                       default=False,
                       dest="progress",
                       help="Only prints the progress of the simulation, but swallows all the other output")
        grp.add_option("--logname",
                       dest="logname",
                       default=logname,
                       help="Name of the logfile")
        grp.add_option("--compress",
                       action="store_true",
                       dest="compress",
                       default=False,
                       help="Compress the logfile into a gzip file. Possible loss of data if the run fails")
        grp.add_option("--no-log",
                       action="store_true",
                       dest="noLog",
                       default=False,
                       help="Do not output a log-file")

        self.parser.add_option_group(grp)

        inf=OptionGroup(self.parser,
                        "Run Info",
                        "Additional information about the run")
        inf.add_option("--remark",
                       dest="remark",
                       default=None,
                       help="Text string with a remark about the run")
        inf.add_option("--job-id",
                       dest="jobId",
                       default=None,
                       help="Text string with the job-ID of the queuing system (usually unused)")
        self.parser.add_option_group(inf)

    def setLogname(self,
                   default="PyFoamRunner",
                   useApplication=True):
        """Builds a logfile-name
        @param default: Default value if no prefix for the logfile-has been defined
        @param useApplication: append the name of the application to the prefix"""

        if self.opts.logname==None:
            self.opts.logname=default
        if useApplication:
            self.opts.logname+="."+path.basename(self.parser.getArgs()[0])
            
