#  ICE Revision: $Id: /local/openfoam/Python/PyFoam/PyFoam/Execution/AnalyzedRunner.py 5894 2009-11-27T14:09:20.728939Z bgschaid  $ 
"""Command is run and output is analyzed"""

from BasicRunner import BasicRunner
from StepAnalyzedCommon import StepAnalyzedCommon

class AnalyzedRunner(StepAnalyzedCommon,BasicRunner):
    """The output of a command is analyzed while being run

    Side effects (files written etc) depend on the analyzer"""
    
    def __init__(self,analyzer,
                 argv=None,
                 silent=False,
                 logname="PyFoamSolve",
                 server=False,
                 lam=None,
                 compressLog=False,
                 restart=False,
                 noLog=False,
                 remark=None,
                 jobId=None,
                 smallestFreq=60.):
        """ @param analyzer: the analyzer for the output
        argv, silent, logname, server, lam, noLog - see BasicRunner"""
        BasicRunner.__init__(self,argv,silent,logname,
                             server=server,
                             lam=lam,
                             restart=restart,
                             compressLog=compressLog,
                             noLog=noLog,
                             remark=remark,
                             jobId=jobId)
        StepAnalyzedCommon.__init__(self,
                                    logname,
                                    analyzer,
                                    smallestFreq=smallestFreq)

    def lineHandle(self,line):
        """Not to be called: calls the analyzer for the current line"""
        StepAnalyzedCommon.lineHandle(self,line)
        BasicRunner.lineHandle(self,line)

    def lastTime(self):
        return self.getTime()

    def firstCpuTime(self):
        exe=self.getAnalyzer("Execution")
        if exe==None:
            return None
        else:
            return exe.timeFirst()

    def firstClockTime(self):
        exe=self.getAnalyzer("Execution")
        if exe==None:
            return None
        else:
            return exe.clockFirst()        

    def totalCpuTime(self):
        exe=self.getAnalyzer("Execution")
        if exe==None:
            return None
        else:
            return exe.timeTotal()

    def totalClockTime(self):
        exe=self.getAnalyzer("Execution")
        if exe==None:
            return None
        else:
            return exe.clockTotal()

    def stopHandle(self):
        BasicRunner.stopHandle(self)
        StepAnalyzedCommon.stopHandle(self)

        self.tearDown()
            
